@@ -611,7 +611,11 @@ client_save_yourself_callback (SmcConn   smc_conn,
   gboolean ret;
 
   if (!client_grab_widget)
-    client_grab_widget = gtk_widget_new (GTK_TYPE_INVISIBLE, NULL);
+    {
+      GDK_THREADS_ENTER();
+      client_grab_widget = gtk_widget_new (GTK_TYPE_INVISIBLE, NULL);
+      GDK_THREADS_LEAVE();
+    }
 
   /* The first SaveYourself after registering for the first time
    * is a special case (SM specs 7.2).
@@ -689,6 +693,8 @@ client_save_yourself_callback (SmcConn   smc_conn,
 
   client_set_state (client, GNOME_CLIENT_SAVING_PHASE_1);
 
+  GDK_THREADS_ENTER();
+
   if (gdk_pointer_is_grabbed())
     {
       gboolean waiting = TRUE;
@@ -702,12 +708,17 @@ client_save_yourself_callback (SmcConn   smc_conn,
   /* Check that we did not receive a shutdown cancelled while waiting
    * for the grab to be released. The grab should prevent it but ... */
   if (client->state != GNOME_CLIENT_SAVING_PHASE_1)
-    return;
+    {
+      GDK_THREADS_LEAVE();
+      return;
+    }
 
   gdk_pointer_ungrab (GDK_CURRENT_TIME);
   gdk_keyboard_ungrab (GDK_CURRENT_TIME);
   gtk_grab_add (client_grab_widget);
 
+  GDK_THREADS_LEAVE();
+
   name = g_strdup (gnome_client_get_global_config_prefix(client));
   name[strlen (name) - 1] = '\0';
 
@@ -768,7 +779,11 @@ client_die_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client= (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[DIE], 0);
 }
@@ -780,7 +795,11 @@ client_save_complete_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client = (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[SAVE_COMPLETE], 0);
 }
@@ -792,7 +811,11 @@ client_shutdown_cancelled_callback (SmcConn smc_conn, SmPointer client_data)
   GnomeClient *client= (GnomeClient*) client_data;
 
   if (client_grab_widget)
-    gtk_grab_remove (client_grab_widget);
+    {
+      GDK_THREADS_ENTER();
+      gtk_grab_remove (client_grab_widget);
+      GDK_THREADS_LEAVE();
+    }
 
   g_signal_emit (client, client_signals[SHUTDOWN_CANCELLED], 0);
 }
@@ -846,6 +869,22 @@ static void client_parse_func (poptContext ctx,
 			       enum poptCallbackReason reason,
 			       const struct poptOption *opt,
 			       const char *arg, void *data);
+
+static gboolean gnome_client_goption_sm_client_id (const gchar *option_name,
+						   const gchar *value,
+						   gpointer data,
+						   GError **error);
+
+static gboolean gnome_client_goption_sm_config_prefix (const gchar *option_name,
+						       const gchar *value,
+						       gpointer data,
+						       GError **error);
+
+static gboolean gnome_client_goption_sm_disable (const gchar *option_name,
+						 const gchar *value,
+						 gpointer data,
+						 GError **error);
+
 static void gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info);
 static void gnome_client_post_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info);
 
@@ -869,6 +908,22 @@ static const struct poptOption options[] = {
   {NULL, '\0', 0, NULL, 0}
 };
 
+static const GOptionEntry session_goptions[] = {
+  { "sm-client-id", '\0', 0, G_OPTION_ARG_CALLBACK,
+    gnome_client_goption_sm_client_id, 
+    N_("Specify session management ID"), N_("ID")},
+
+  { "sm-config-prefix", '\0', 0, G_OPTION_ARG_CALLBACK,
+    gnome_client_goption_sm_config_prefix, 
+    N_("Specify prefix of saved configuration"), N_("PREFIX")},
+
+  { "sm-disable", '\0', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
+    gnome_client_goption_sm_disable, 
+    N_("Disable connection to session manager"), NULL},
+
+  { NULL }
+};
+
 typedef struct {
 	guint sm_connect_id;
 } GnomeProgramClass_gnome_client;
@@ -966,6 +1021,21 @@ gnome_client_module_instance_init (GnomeProgram *program, GnomeModuleInfo *mod_i
     g_object_set_qdata (G_OBJECT (program), quark_gnome_program_private_gnome_client, priv);
 }
 
+static GOptionGroup *
+gnome_client_module_get_goption_group (void)
+{
+	GOptionGroup *option_group;
+
+	option_group = g_option_group_new ("gnome-session",
+					   N_("Session management"),
+					   N_("Show session management options"),
+					   NULL, NULL);
+	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+	g_option_group_add_entries(option_group, session_goptions);
+
+	return option_group;
+}
+
 const GnomeModuleInfo *
 gnome_client_module_info_get (void)
 {
@@ -978,6 +1048,8 @@ gnome_client_module_info_get (void)
 		NULL, NULL
 	};
 
+	module_info.expansion1 = gnome_client_module_get_goption_group;
+
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[3];
 
@@ -1028,6 +1100,41 @@ client_parse_func (poptContext ctx,
   }
 }
 
+static gboolean
+gnome_client_goption_sm_client_id (const gchar *option_name,
+				   const gchar *value,
+				   gpointer data,
+				   GError **error)
+{
+	gnome_client_set_id (master_client, value);
+
+	return TRUE;
+}
+
+static gboolean
+gnome_client_goption_sm_disable (const gchar *option_name,
+				 const gchar *value,
+				 gpointer data,
+				 GError **error)
+{
+	g_object_set (G_OBJECT (gnome_program_get()),
+		      GNOME_CLIENT_PARAM_SM_CONNECT, FALSE, NULL);
+
+	return TRUE;
+}
+
+static gboolean
+gnome_client_goption_sm_config_prefix (const gchar *option_name,
+				       const gchar *value,
+				       gpointer data,
+				       GError **error)
+{
+	g_free (master_client->config_prefix);
+	master_client->config_prefix = g_strdup (value);
+	master_client_restored = TRUE;
+ 
+	return TRUE;
+}
 
 static void
 gnome_client_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
