@@ -68,6 +68,8 @@
 #include <unistd.h>
 #include <string.h>
 
+#include <glib/gstdio.h>
+
 struct _GnomeIconEntryPrivate {
 	GtkWidget *fentry;
 	char *picked_file;
@@ -388,7 +390,7 @@ entry_activated(GtkWidget *widget, GnomeIconEntry *ientry)
 	if (!filename)
 		return;
 
-	stat (filename, &buf);
+	g_stat (filename, &buf);
 	if (S_ISDIR (buf.st_mode)) {
 		gnome_icon_selection_clear (GNOME_ICON_SELECTION (priv->icon_sel), TRUE);
 		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (priv->icon_sel), filename);
@@ -406,7 +408,7 @@ entry_activated(GtkWidget *widget, GnomeIconEntry *ientry)
 static void
 setup_preview(GtkWidget *widget)
 {
-	const gchar *p;
+	char *p;
 	GList *l;
 	GtkWidget *pp = NULL;
         GdkPixbuf *pixbuf, *scaled;
@@ -430,8 +432,12 @@ setup_preview(GtkWidget *widget)
 
 	p = gtk_file_chooser_get_filename(fc);
 	if(!p || !g_file_test (p,G_FILE_TEST_IS_SYMLINK | G_FILE_TEST_IS_REGULAR) ||
-	   !(pixbuf = gdk_pixbuf_new_from_file (p, NULL)))
+	   !(pixbuf = gdk_pixbuf_new_from_file (p, NULL))) {
+		g_free (p);
 		return;
+	}
+
+	g_free (p);
 
 	w = gdk_pixbuf_get_width(pixbuf);
 	h = gdk_pixbuf_get_height(pixbuf);
@@ -561,6 +567,7 @@ icon_selected_cb (GnomeIconEntry * ientry)
 	icon = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY (ientry->_priv->fentry), FALSE);
 	
 	if (icon != NULL) {
+		g_free (priv->picked_file);
 		priv->picked_file = icon;
 		update_icon (ientry);
 		g_signal_emit (ientry, gnome_ientry_signals[CHANGED_SIGNAL], 0);
@@ -603,6 +610,7 @@ gil_icon_selected_cb(GnomeIconList *gil, gint num, GdkEvent *event, GnomeIconEnt
 	    event->type == GDK_2BUTTON_PRESS && ((GdkEventButton *)event)->button == 1) {
 		gnome_icon_selection_stop_loading
 			(GNOME_ICON_SELECTION (priv->icon_sel));
+		g_free (priv->picked_file);
 		priv->picked_file = icon;
 		icon = NULL;
 		update_icon (ientry);
@@ -818,7 +826,7 @@ drag_data_received (GtkWidget        *widget,
 	GnomeVFSFileInfo *file_info;
 	int i;
 
-	uris = g_strsplit (selection_data->data, "\r\n", -1);
+	uris = g_strsplit ((char *)selection_data->data, "\r\n", -1);
 	if (uris == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
@@ -952,7 +960,7 @@ drag_data_get  (GtkWidget          *widget,
 	g_free(file);
 	gtk_selection_data_set (selection_data,
 				selection_data->target,
-				8, string, strlen(string)+1);
+				8, (unsigned char *)string, strlen(string)+1);
 	g_free(string);
 }
 
@@ -998,9 +1006,6 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 	g_signal_connect (ientry->_priv->pickbutton, "clicked",
 			  G_CALLBACK (show_icon_selection),
 			  ientry);
-	/*FIXME: 60x60 is just larger then default 48x48, though icon sizes
-	  are supposed to be selectable I guess*/
-	gtk_widget_set_size_request (ientry->_priv->pickbutton, 60, 60);
 	gtk_container_add (GTK_CONTAINER (w), ientry->_priv->pickbutton);
 	gtk_widget_show (ientry->_priv->pickbutton);
 
