@@ -80,6 +80,10 @@ static void libgnomeui_arg_callback 	(poptContext con,
 					 const struct poptOption * opt,
 					 const char * arg,
 					 void * data);
+static gboolean libgnomeui_goption_disable_crash_dialog (const gchar *option_name,
+							 const gchar *value,
+							 gpointer data,
+							 GError **error);
 static void libgnomeui_init_pass	(const GnomeModuleInfo *mod_info);
 static void libgnomeui_class_init	(GnomeProgramClass *klass,
 					 const GnomeModuleInfo *mod_info);
@@ -124,6 +128,29 @@ static struct poptOption libgnomeui_options[] = {
 	{NULL, '\0', 0, NULL, 0, NULL, NULL}
 };
 
+static GOptionEntry libgnomeui_goptions[] = {
+	{ "disable-crash-dialog", '\0', G_OPTION_FLAG_NO_ARG,
+	  G_OPTION_ARG_CALLBACK, libgnomeui_goption_disable_crash_dialog,
+	  N_("Disable Crash Dialog"), NULL },
+	/* --display is already handled by gtk+ */
+	{ NULL }
+};
+
+static GOptionGroup *
+libgnomeui_get_goption_group (void)
+{
+	GOptionGroup *option_group;
+
+	option_group = g_option_group_new ("gnome-ui",
+					   N_("GNOME GUI Library"),
+					   N_("Show GNOME GUI options"),
+					   NULL, NULL);
+	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+	g_option_group_add_entries(option_group, libgnomeui_goptions);
+
+	return option_group;
+}
+
 const GnomeModuleInfo *
 libgnomeui_module_info_get (void)
 {
@@ -136,6 +163,8 @@ libgnomeui_module_info_get (void)
 		NULL, NULL
 	};
 
+	module_info.expansion1 = libgnomeui_get_goption_group;
+
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[6];
 
@@ -497,8 +526,10 @@ event_sounds_changed_cb (GConfClient* client, guint cnxn_id, GConfEntry *entry,
                                 gnome_gconf_get_bool ("/desktop/gnome/sound/event_sounds"));
 
         if (new_use_event_sounds && !use_event_sounds) {
+                GDK_THREADS_ENTER();
                 initialize_gtk_signal_relay ();
                 initialize_gnome_signal_relay ();
+                GDK_THREADS_LEAVE();
 	}
 
         use_event_sounds = new_use_event_sounds;
@@ -593,6 +624,18 @@ libgnomeui_arg_callback(poptContext con,
         }
 }
 
+static gboolean
+libgnomeui_goption_disable_crash_dialog (const gchar *option_name,
+					 const gchar *value,
+					 gpointer data,
+					 GError **error)
+{ 
+	g_object_set (G_OBJECT (gnome_program_get ()),
+		      LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE, NULL);
+
+	return TRUE;
+}
+
 /* automagically parse all the gtkrc files for us.
  * 
  * Parse:
